/*
 This file is part of iDQ.

 iDQ is an instantiation-based DQBF (Dependency Quantified Boolean Formula) solver.

 Andreas Froehlich, Johannes Kepler University, Linz, Austria, 2012-2014.
 Gergely Kovasznai, Vienna University of Technology, Vienna, Austria, 2012-2014.

 iDQ is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or (at
 your option) any later version.

 iDQ is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with iDQ.  If not, see <http://www.gnu.org/licenses/>.
*/

/*
 * Additional information to the Boolean clause set generated from the original clause set
 */

#ifndef BFORMULA_H_INCLUDED
#define BFORMULA_H_INCLUDED

#include "idq.h"
#include "stack.h"
#include "hashtable.h"
#include "instlist.h"

/*
 * Options:
 * 		- parameters for VSIDS
 */

#define VARSCORE_TYPE float
// in how many periods the var scores should be divided
#define VSIDS_PERIOD 50
// the value of the divisor
#define VSIDS_DIVISOR 2

#include "abformula.h"

// Boolean variable
typedef struct BVariable {
	VarID varId;

	VARSCORE_TYPE score; // var score
#ifdef INST_VARSCORE_ON_THE_FLY
	InstLitRefStack varWatchers; // list of passive instances refer to this (Boolean) var
#endif
} BVariable;

// Boolean clause
typedef struct BClause {
	unsigned int cntLits;
	BVariable ** pVars; // (Boolean) vars in this clause (signs are given by the original clause)
} BClause;

// get the Boolean literal at the index "litInd" in the given instance
#define GET_BLIT_FROM_INST(abClause,instInd,litInd) (((abClause)->lits[(litInd)] > 0 ? 1 : -1) * GET_INST_BY_ABCLAUSE_AND_INSTIND((abClause),(instInd))->bClause->pVars[(litInd)]->varId)

DECLARE_STACK(BVar, BVariable*);

// main structure
typedef struct BFormula {
	unsigned int cntClauses; // overall count of Boolean clauses

	BVarStack pVars; // variables that have been pushed to the SAT solver
	HashTable * pHashTable; // hashtable that maps var instances to Boolean vars
} BFormula;

struct iDQ;

void create_bFormula(struct iDQ *);

void delete_bClause(iDQMemMan *, BClause *);
void delete_bFormula(struct iDQ *, BFormula *);


//#include "satsolver.h"

struct AbFormula;
struct AbClause;
struct AbClauseInst;

void print_satModel(struct iDQ *);

BVariable * get_bVar_fromVarInst(struct iDQ *, unsigned int, IList *);

BClause * convert_instToBClause(struct iDQ *, struct AbClause *, unsigned int);

void add_inst_toSatSolver(iDQ *, struct AbClause *, unsigned int);

Result getValue_lit(struct AbClause *, unsigned int, unsigned int, SatSolver *);
Result getValue_inst(struct AbClause *, unsigned int, SatSolver *);

void decrease_varScores(iDQ *);
#define UPDATE_VAR_SCORES(idq) if (!((idq)->clock % VSIDS_PERIOD)) decrease_varScores((idq));

unsigned int select_satLiteral(struct iDQ *, struct AbClause *, unsigned int);

#endif
