/*
 This file is part of iDQ.

 iDQ is an instantiation-based DQBF (Dependency Quantified Boolean Formula) solver.

 Andreas Froehlich, Johannes Kepler University, Linz, Austria, 2012-2014.
 Gergely Kovasznai, Vienna University of Technology, Vienna, Austria, 2012-2014.

 iDQ is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or (at
 your option) any later version.

 iDQ is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with iDQ.  If not, see <http://www.gnu.org/licenses/>.
*/

/*
 * Hash table data structure
 */

#ifndef HASHTABLE_H_INCLUDED
#define HASHTABLE_H_INCLUDED

#include "idq.h"

/*
 * Function pointer type for hash functions
 */
typedef unsigned (*HashPtr)(const void *key);

/*
 * Type for data assigned to keys
 */
typedef union HashData {
	int asInt;
	void * asPtr;
//  char * asStr;
} HashData;

/*
 * Type for slots (buckets) in the hashtable
 */
typedef struct HashBucket {
	/* public:
	 */
	void *key;

	HashData data;

	struct HashBucket *next; /* chronologically */
	struct HashBucket *prev; /* chronologically */

	/* private:
	 */
	struct HashBucket *chain;/* collision chain */
} HashBucket;


/*
 * Function pointer types used by other data structures
 */
typedef int (*CmpPtr)(const void *a, const void *b);
typedef void (*DeletePtr)(iDQ *qpr, void *data);

/*
 * Type for hashtables
 */
typedef struct HashTable {
	iDQ *qpr;

	unsigned size;
	unsigned count;
	HashBucket **table;

	HashPtr hash;
	CmpPtr cmp;
	DeletePtr del;

	HashBucket *first;/* chronologically */
	HashBucket *last; /* chronologically */
} HashTable;

HashTable *create_hashTable(iDQ *, HashPtr, CmpPtr, DeletePtr);

void delete_hashTable(HashTable *);

HashBucket * find_hashTable(HashTable *, void *);

HashBucket * insert_hashTable(HashTable *, void *key);
void remove_hashTable(HashTable *, void * key, void ** stored_key_ptr, HashData * stored_data_ptr);

#endif
