/*
 This file is part of iDQ.

 iDQ is an instantiation-based DQBF (Dependency Quantified Boolean Formula) solver.

 Andreas Froehlich, Johannes Kepler University, Linz, Austria, 2012-2014.
 Gergely Kovasznai, Vienna University of Technology, Vienna, Austria, 2012-2014.

 iDQ is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or (at
 your option) any later version.

 iDQ is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with iDQ.  If not, see <http://www.gnu.org/licenses/>.
*/

/*
 * Literal (LitID) list, realized by bit array
 */

#ifndef INSTLIST_H_
#define INSTLIST_H_

#include <stdio.h>

#include "idq.h"
#include "mem.h"

#define UNSIGNED 0
#define SIGNED 1

typedef struct IList {
	iDQMemMan *mem;

	unsigned int numOfVars; // the size of the domain, in bits
	unsigned int numOfVarsInInts; // the size of the domain, in ints
	unsigned int* array; // the current domain of the variable, represented by a bit array
	char isSigned; // if literal are not signed, the list contains variables
} IList;

/*
 * For a uint that holds variables, returns a uint that holds their signs
 */
#define GET_SIGN_UINT_FOR_VAR_UINT(iList, pVarUint) *(pVarUint + iList->numOfVarsInInts)

IList* create_iList(iDQMemMan * mem, unsigned int numOfVars, char isSigned);
void delete_iList(void* _iList);

LitID getLit_iList(IList* iList, unsigned int varId);
void setLit_iList(IList* iList, LitID litId);

void intersectWith_iList(IList* iList1, IList* iList2);
IList * intersection_iList(IList* iList1, IList* iList2);
IList * union_iList(IList* iList1, IList* iList2);

char equal_iList(IList* iList1, IList* iList2, char isSigned);
char subseteq_iList(IList* iList1, IList* iList2, char isSigned);

unsigned int countLits_iList(IList* iList);

void print_iList(IList* iList, FILE* out);

#endif
