/*
 This file is part of iDQ.

 iDQ is an instantiation-based DQBF (Dependency Quantified Boolean Formula) solver.

 Andreas Froehlich, Johannes Kepler University, Linz, Austria, 2012-2014.
 Gergely Kovasznai, Vienna University of Technology, Vienna, Austria, 2012-2014.

 iDQ is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or (at
 your option) any later version.

 iDQ is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with iDQ.  If not, see <http://www.gnu.org/licenses/>.

 This file incorporates work covered by the following copyright and
 permission notice:  

   DepQBF
   Copyright 2010-2014 Florian Lonsing, Johannes Kepler University, Linz, Austria.
   Copyright 2012 Aina Niemetz, Johannes Kepler University, Linz, Austria.
*/

/*
 * Memory manager
 */

#ifndef IDQ_MEM_H_INCLUDED
#define IDQ_MEM_H_INCLUDED

#include <stddef.h>

#define NEWN(mm,ptr,nelems) \
  do { (ptr) = idq_malloc ((mm), (nelems) * sizeof *(ptr)); } while (0)

#define NEW(mm,ptr) NEWN((mm),(ptr),1)

#define CNEWN(mm,ptr,nelems) \
  do { (ptr) = idq_calloc ((mm), (nelems), sizeof *(ptr)); } while (0)

#define CNEW(mm,ptr) CNEWN((mm),(ptr),1)

#define REALLOC(mm,ptr,nelems1,nelems2) \
  do { (ptr) = idq_realloc ((mm), (ptr), (nelems1) * sizeof *(ptr), (nelems2) * sizeof *(ptr)); } while (0)

#define CLRN(ptr,nelems) \
  do { memset ((ptr), 0,  (nelems) * sizeof *(ptr)); } while (0)

#define CLR(ptr) CLRN((ptr),1)

#define DELETEN(mm,ptr,nelems) \
  do { idq_free ((mm), (ptr), (nelems) * sizeof *(ptr)); } while (0)

#define DELETE(mm,ptr) DELETEN((mm),(ptr),1)

typedef struct iDQMemMan {
	size_t cur_allocated;
	size_t max_allocated;
	size_t limit;
} iDQMemMan;

iDQMemMan *idq_create_mem_man();

void idq_delete_mem_man(iDQMemMan * mm);

void *idq_malloc(iDQMemMan * mm, size_t size);

void *idq_calloc(iDQMemMan * mm, size_t nobj, size_t size);

void *idq_realloc(iDQMemMan * mm, void *ptr, size_t old_size, size_t new_size);

void idq_free(iDQMemMan * mm, void *ptr, size_t size);

size_t idq_max_allocated(iDQMemMan * mm);

size_t idq_cur_allocated(iDQMemMan * mm);

void idq_set_mem_limit(iDQMemMan * mm, size_t limit);

size_t idq_get_mem_limit(iDQMemMan * mm);

#endif
