/*
 This file is part of iDQ.

 iDQ is an instantiation-based DQBF (Dependency Quantified Boolean Formula) solver.

 Andreas Froehlich, Johannes Kepler University, Linz, Austria, 2012-2014.
 Gergely Kovasznai, Vienna University of Technology, Vienna, Austria, 2012-2014.

 iDQ is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or (at
 your option) any later version.

 iDQ is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with iDQ.  If not, see <http://www.gnu.org/licenses/>.
*/

/*
 * Priority queue data structure for passive instances of clauses
 */

#ifndef PRIOR_QUEUE_H_
#define PRIOR_QUEUE_H_

#include "idq.h"
#include "abformula.h"

// reference to a passive instance
typedef struct PassiveInstRef {
	InstRef instRef;

	unsigned int createdAt; // when was it created
	unsigned int numDeps, numSymbs, numLits; // data for heuristics
} PassiveInstRef;

// compute the heuristic data for a PassiveInstRef
#define COMPUTE_PASSIVE_INST_HEUR_DATA(passiveInstRef)		\
{		\
	(passiveInstRef)->numSymbs = countLits_iList(GET_INST_BY_INSTREF((passiveInstRef)->instRef)->instMask);		\
	(passiveInstRef)->numDeps = countLits_iList((passiveInstRef)->instRef.abClause->depVarMask) - (passiveInstRef)->numSymbs;		\
	(passiveInstRef)->numLits = (passiveInstRef)->instRef.abClause->cntLits;		\
}

struct AbFormula;

typedef struct PriorQueue {
	iDQMemMan *mem;
	struct AbFormula * abf;

	unsigned size;
	unsigned count;
	PassiveInstRef ** list;

	CmpPtr cmp;
} PriorQueue;

PriorQueue *create_priorQueue(iDQMemMan *, struct AbFormula *, CmpPtr);

void delete_priorQueue(PriorQueue *);

unsigned int enqueue_priorQueue(PriorQueue *, PassiveInstRef *);
PassiveInstRef *dequeue_priorQueue(PriorQueue *);
unsigned int rearrangeElement_priorQueue(PriorQueue *, unsigned int);

#endif
