/*
 This file is part of iDQ.

 iDQ is an instantiation-based DQBF (Dependency Quantified Boolean Formula) solver.

 Andreas Froehlich, Johannes Kepler University, Linz, Austria, 2012-2014.
 Gergely Kovasznai, Vienna University of Technology, Vienna, Austria, 2012-2014.

 iDQ is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or (at
 your option) any later version.

 iDQ is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with iDQ.  If not, see <http://www.gnu.org/licenses/>.

 This file incorporates work covered by the following copyright and
 permission notice:  

   DepQBF
   Copyright 2010-2014 Florian Lonsing, Johannes Kepler University, Linz, Austria.
   Copyright 2012 Aina Niemetz, Johannes Kepler University, Linz, Austria.
*/

/*
 * Methods to parse and dump DQDIMACS format, and data structure for original clause set
 */

#ifndef QBFORMULA_H_INCLUDED
#define QBFORMULA_H_INCLUDED

#include "idq.h"

struct Clause;
struct Variable;

/*
 * The original clause set
 */
typedef struct QBFormula {
	unsigned int cntVars;
	unsigned int cntClauses;
	unsigned int cntDeletedClauses;

	unsigned int cntUniversals;
	unsigned int cntExistentials;

	struct Clause * pClauses;
	struct Variable * pVars;
} QBFormula;

void delete_qbFormula(iDQMemMan *, QBFormula *);

void parse_dqdimacs(iDQ *);

void dump_dqdimacs(iDQ *);
void dump_tptp(iDQ *);

#endif
