/*
 This file is part of iDQ.

 iDQ is an instantiation-based DQBF (Dependency Quantified Boolean Formula) solver.

 Andreas Froehlich, Johannes Kepler University, Linz, Austria, 2012-2014.
 Gergely Kovasznai, Vienna University of Technology, Vienna, Austria, 2012-2014.

 iDQ is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or (at
 your option) any later version.

 iDQ is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with iDQ.  If not, see <http://www.gnu.org/licenses/>.
*/

/*
 * Interface for SAT solvers
 */

#ifndef SATSOLVER_H_INCLUDED
#define SATSOLVER_H_INCLUDED

#include "stdint.h"

#include "lglib.h"
#include "picosat.h"

/*
 * Options:
 * 		- which SAT solver to use
 */

// which SAT solver to use
#define SAT_SOLVER_PICOSAT
//#define SAT_SOLVER_LINGELING

/*
 * Interface to PicoSAT
 */
#ifdef SAT_SOLVER_PICOSAT
typedef PicoSAT SatSolver;
#define INIT_SAT_SOLVER picosat_init
#define RELEASE_SAT_SOLVER(s) picosat_reset(s)
#define ADD_LIT_TO_SAT_SOLVER(s,l) picosat_add(s,l)
#define CHECK_SAT_WITH_SAT_SOLVER(s) picosat_sat(s,-1)
#define LIT_VALUE_BY_SAT_SOLVER(s,l) picosat_deref(s,l)
#endif

/*
 * Interface to Lingeling
 */
#ifdef SAT_SOLVER_LINGELING
typedef LGL SatSolver;
#define INIT_SAT_SOLVER lglinit
#define RELEASE_SAT_SOLVER(s) lglrelease(s)
#ifdef INCREMENTAL_SAT
#define ADD_LIT_TO_SAT_SOLVER(s,l) if ((l)) lglfreeze(s,l); lgladd(s,l)
#else
#define ADD_LIT_TO_SAT_SOLVER(s,l) lgladd(s,l)
#endif
#define CHECK_SAT_WITH_SAT_SOLVER(s) lglsat(s)
#define LIT_VALUE_BY_SAT_SOLVER(s,l) lglderef(s,l)
#endif

struct BClause;

#endif

