/*
 This file is part of iDQ.

 iDQ is an instantiation-based DQBF (Dependency Quantified Boolean Formula) solver.

 Andreas Froehlich, Johannes Kepler University, Linz, Austria, 2012-2014.
 Gergely Kovasznai, Vienna University of Technology, Vienna, Austria, 2012-2014.

 iDQ is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or (at
 your option) any later version.

 iDQ is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with iDQ.  If not, see <http://www.gnu.org/licenses/>.

 This file incorporates work covered by the following copyright and
 permission notice:  

   DepQBF
   Copyright 2010-2014 Florian Lonsing, Johannes Kepler University, Linz, Austria.
   Copyright 2012 Aina Niemetz, Johannes Kepler University, Linz, Austria.
*/

/*
 * Stack data structure
 */

#ifndef STACK_H_INCLUDED
#define STACK_H_INCLUDED

#include "mem.h"

#define DECLARE_STACK(name, type)					\
  typedef struct name ## Stack name ## Stack;				\
  struct name ## Stack { iDQMemMan * mem; type * start; type * top; type * end; }

#define INIT_STACK(mm, stack)			   \
  {						\
	(stack).mem = mm;				\
    (stack).start = (stack).top = (stack).end = 0; \
  }
#define COUNT_STACK(stack) ((size_t) ((stack).top - (stack).start))
#define EMPTY_STACK(stack) ((stack).top == (stack).start)
#define RESET_STACK(stack) do { (stack).top = (stack).start; } while (0)

#define SIZE_STACK(stack) ((stack).end - (stack).start)
#define FULL_STACK(stack) ((stack).top == (stack).end)
#define DELETE_STACK(stack)					\
  do {				\
	if ((stack).start) DELETEN((stack).mem, (stack).start, SIZE_STACK(stack));			\
    INIT_STACK ((stack).mem, (stack));						\
  } while (0)
#define ENLARGE_STACK(stack)					\
  {									\
    size_t old_size = SIZE_STACK (stack), new_size;		\
    new_size = old_size ? 2 * old_size : 1;				\
    size_t old_count = COUNT_STACK (stack);			\
    size_t elem_bytes = sizeof(*(stack).start);				\
    REALLOC((stack).mem, (stack).start,			\
				  old_size*elem_bytes,			\
				  new_size*elem_bytes);			\
    (stack).top = (stack).start + old_count;				\
    (stack).end = (stack).start + new_size;				\
  }

#define PUSH_STACK(stack, elem)	\
  {						\
    if (FULL_STACK ((stack)))		\
      ENLARGE_STACK ((stack));	\
    *((stack).top++) = (elem);			\
  }

#define TOP_STACK(stack) ((stack).top - 1)

#define POP_STACK(stack) (*--(stack).top)

#define GET_STACK(stack, index) ((stack).start + (index))

/*
 * Print ALL the elements of a stack.
 * 		printFunction: pointer to the function that prints an element
 */
#define PRINT_STACK(dqbf, stack, printFunction)					\
  {									\
  	size_t i, count;				\
  	for (i = 0, count = COUNT_STACK(stack); i<count; i++)			\
			(*printFunction)(dqbf, *((stack).start + i));						\
  }

#endif
